/*******************************************************************************
*    Comm2J 1.0.                                                               *
*    Copyright (C) 2004 Luca Bertoncello                                       *
*    Hartigstrae, 12 - 01127 Dresden Deutschland                              *
*    E-Mail: lucabert@lucabert.com, lucabert@lucabert.de                       *
*    http://www.lucabert.com/  http://www.lucabert.de/                         *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU General Public License as published by      *
*    the Free Software Foundation; version 2 of the License.                   *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU General Public License for more details.                              *
*                                                                              *
*    You should have received a copy of the GNU General Public License         *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

package comm2j;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.IOException;

import comm2j.Message;
import comm2j.LZW2J;

/**
 * Read2J is a class used to communicate with another peer (J2ME or J2SE), transmitting
 * and receiving Messages throws an opened Socket connection.<br>
 * Read2J supports compression (<b>LZW</b>), too. This means that you can send and
 * receive <u>transparently</u> data through a compression tunnel.<br>
 * It's possible to have just a compressed way (eg: from server to client), so if
 * you have to send little quantity of data (that cannot be compressed), you can send
 * them raw.<br>
 * The other peer will automatically recognise if the communication to him is
 * raw or compressed.
 * @see MessageI
 * @author Luca Bertoncello &lt;lucabert@lucabert.de&gt;
 * @version 1.0
*/

public class Read2J
{
  /**
   * Defines a normal communication between peers
   */
  public static final int RAW         = 0;
  /**
   * Defines a compressed (<b>LZW</b>) communication between peers
   */
  public static final int COMPRESSED  = 1;
  /**
   * Defines the number of bytes used to describe the lenght of the packet
   */
  public static final int PACKLENGTH  = 3;

  private int mode                    = Read2J.RAW;

  /**
   * Constructor.
   * Used communication mode will be setted to default (RAW)
   */
  public Read2J()
  {
    this.mode                         = Read2J.RAW;
  }

  /**
   * Returns the mode used by Read2J for the communication
   * @return Currently used communication mode
   */
  public int getMode()
  {
    return this.mode;
  }

  /**
   * Sets the mode used by Read2J for the communication
   * @param mode Communication mode to be used
   */
  public void setMode(int mode)
  {
    this.mode                         = mode;
  }

  /**
   * Sends a message to the remote peer.<br>
   * If compression mode was set, the stream will be compressed.
   * @param msg Message to be sent
   * @param os OutputStream
   * @return number of sent bytes
   * @throws java.io.IOException if an error occoured
   */
  public int writeMessageToStream(Message msg, OutputStream os) throws IOException
  {
    int len                           = -1;

    os.write(this.mode);
    ByteArrayOutputStream baos        = new ByteArrayOutputStream();
    msg.writeHeader(baos);
    msg.writeMessage(baos);

    if(this.mode == Read2J.RAW)
    {
      Read2J.writeLength(os, baos.size());
      os.write(baos.toByteArray());

      len                             = baos.toByteArray().length;
    }
    else if(this.mode == Read2J.COMPRESSED)
    {
      ByteArrayInputStream baisc      = new ByteArrayInputStream(baos.toByteArray());
      ByteArrayOutputStream baosc     = new ByteArrayOutputStream();
      LZW2J lzw2j                     = new LZW2J();
      lzw2j.compress(baisc, baosc);
      Read2J.writeLength(os, baosc.size());
      os.write(baosc.toByteArray());
      len                             = baosc.toByteArray().length;
    }
    os.flush();
    os.close();
    System.gc();

    return len;
  }

  /**
   * Receive a message from the remote peer.<br>
   * It automatically recognises if the communication was
   * compressed or not.
   * @param is InputStream
   * @return Message read from the remote host
   * @throws java.io.IOException if an error occoured
   */
  public Message readMessageFromStream(InputStream is) throws IOException
  {
    DataInputStream dis;
    ByteArrayInputStream bais;
    byte[] inMsg;
    Message msg                       = null;

    int mode                          = is.read();
    int len                           = Read2J.readLength(is);
    inMsg                             = new byte[len];
    for(int l = 0; l < len; l++)
      inMsg[l]                        = (byte) is.read();

    bais                              = new ByteArrayInputStream(inMsg);
    if(mode == Read2J.RAW)
    {
      msg                             = readMessageFromInternalStream(bais);
    }
    else if(mode == Read2J.COMPRESSED)
    {
      ByteArrayOutputStream baosc     = new ByteArrayOutputStream();
      LZW2J lzw2j                     = new LZW2J();
      lzw2j.decompress(bais, baosc);
      ByteArrayInputStream baisc      = new ByteArrayInputStream(baosc.toByteArray());
      msg                             = readMessageFromInternalStream(baisc);
    }
    System.gc();

    return msg;
  }

  /**
   * Receive a message from given internal stream.<br>
   * It will <u>only</u> be used from readMessageFromStream.
   * @param is InputStream
   * @return Message read from the remote host
   * @throws java.io.IOException if an error occoured
   */
  private Message readMessageFromInternalStream(InputStream is) throws IOException
  {
    Message msgRet                = null;

    String msgType                = (new StringMessage()).readHeader(is);

    if(new ByteMessage().getName().equals(msgType))
    {
      msgRet                      = new ByteMessage();
      msgRet.readMessage(is);
    }
    else if(new DateMessage().getName().equals(msgType))
    {
      msgRet                      = new DateMessage();
      msgRet.readMessage(is);
    }
    else if(new ShortMessage().getName().equals(msgType))
    {
      msgRet                      = new ShortMessage();
      msgRet.readMessage(is);
    }
    else if(new IntegerMessage().getName().equals(msgType))
    {
      msgRet                      = new IntegerMessage();
      msgRet.readMessage(is);
    }
    else if(new LongMessage().getName().equals(msgType))
    {
      msgRet                      = new LongMessage();
      msgRet.readMessage(is);
    }
    else if(new StringMessage().getName().equals(msgType))
    {
      msgRet                      = new StringMessage();
      msgRet.readMessage(is);
    }
    else if(new VectorMessage().getName().equals(msgType))
    {
      msgRet                      = new VectorMessage();
      msgRet.readMessage(is);
    }

    return msgRet;
  }

  /**
   * Sends the length of the packet throws given OutputStream.<br>
   * The length will be coded in 3 bytes, so it's possible to
   * use just 7 bits per byte and armor the codes so that LZW can
   * encode/decode them.
   * @param os OutputStream to be used
   * @param len Length to be sent
   * @throws java.io.IOException if an error occours
   * @see LZW2J
   */
  public static void writeLength(OutputStream os, int len) throws IOException
  {
    int lenH                          = len / 128;
    int t                             = len - (lenH * 128);
    int lenM                          = t / 64;
    int lenL                          = t - (lenM * 64);

    os.write((lenL + 32));
    os.write((lenM + 32));
    os.write((lenH + 32));
  }

  /**
   * Gets the length of the packet from given InputStream.<br>
   * The length is coded in 3 bytes, so it's possible to
   * use just 7 bits per byte and armor the codes so that LZW can
   * encode/decode them.
   * @param is InputStream to be used
   * @return Length of the packet
   * @throws java.io.IOException if an error occours
   * @see LZW2J
   */
  public static int readLength(InputStream is) throws IOException
  {
    int lenL                          = ((int) is.read()) - 32;
    int lenM                          = ((int) is.read()) - 32;
    int lenH                          = ((int) is.read()) - 32;
    int len                           = (lenH * 128) + (lenM * 64) + lenL;

    return len;
  }
}
