/*******************************************************************************
*    Comm2J 1.0.                                                               *
*    Copyright (C) 2004 Luca Bertoncello                                       *
*    Hartigstrae, 12 - 01127 Dresden Deutschland                              *
*    E-Mail: lucabert@lucabert.com, lucabert@lucabert.de                       *
*    http://www.lucabert.com/  http://www.lucabert.de/                         *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU General Public License as published by      *
*    the Free Software Foundation; version 2 of the License.                   *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU General Public License for more details.                              *
*                                                                              *
*    You should have received a copy of the GNU General Public License         *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

package comm2j;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.IOException;
import java.lang.ArrayIndexOutOfBoundsException;
import java.util.Enumeration;
import java.util.Vector;

/**
 * VectorMessage is the message class used to send and receive vector of messages
 * to/from remote server.<br>
 * @see Comm2J
 * @see java.util.Vector
 * @author Luca Bertoncello &lt;lucabert@lucabert.de&gt;
 * @version 1.0
*/

public class VectorMessage extends Message
{
  /**
   * Constructor.
   * Initialize the class with an empty vector
   */
  public VectorMessage()
  {
    super.className                 = "VectorMessage.class";
    super.message                   = new Vector();
  }

  /**
   * Constructor.
   * Initialize the class with given vector
   * @param message Initial vector
   */
  public VectorMessage(Vector message)
  {
    super.className                 = "VectorMessage.class";
    super.messageHasBeenSet         = true;
    super.message                   = (Object) message;
  }

  /**
   * Insert a given message at the end of the vector
   * @param message Given message to be inserted
   */
  public void addMessage(Message message)
  {
    ((Vector) super.message).addElement(message);
  }

  /**
   * Insert a given message at the given position of the vector,
   * replacing old message
   * @param message Given message to be inserted
   * @param index Position of the message
   * @throws java.lang.ArrayIndexOutOfBoundsException if the index was invalid
   */
  public void insertMessageAt(Message message, int index) throws ArrayIndexOutOfBoundsException
  {
    ((Vector) super.message).insertElementAt(message, index);
  }

  /**
   * Returns an enumeration of the messages of this vector.
   * The returned Enumeration object will generate all items in this vector. The first item generated is the item at index 0, then the item at index 1, and so on
   * @return an enumeration of the components of this vector
   */
  public Enumeration elements()
  {
    return ((Vector) super.message).elements();
  }

  /**
   * Returns the number of messages in this vector
   * @return The number of messages in this vector
   */
  public int size()
  {
    return ((Vector) super.message).size();
  }

  /**
   * Reads a message from give InputStream.
   * @param is The InputStream used to read data
   * @return Number of read bytes
   * @throws java.io.IOException if an error occoured
   */
  public int readMessage(InputStream is) throws IOException
  {
    DataInputStream dis;

    super.messageHasBeenSet         = false;
    dis                             = new DataInputStream(is);
    int len                         = Read2J.readLength(dis);
    int read                        = 0;
    ((Vector) super.message).removeAllElements();
    for(int l = 0; l < len; l++)
    {
      String msgType                = readHeader(is);
      if(new ByteMessage().getName().equals(msgType))
      {
        Message msg                 = new ByteMessage();
        read                       += msg.readMessage(is);
        ((Vector) super.message).addElement(msg);
      }
      else if(new DateMessage().getName().equals(msgType))
      {
        Message msg                 = new DateMessage();
        read                       += msg.readMessage(is);
        ((Vector) super.message).addElement(msg);
      }
      else if(new ShortMessage().getName().equals(msgType))
      {
        Message msg                 = new ShortMessage();
        read                       += msg.readMessage(is);
        ((Vector) super.message).addElement(msg);
      }
      else if(new IntegerMessage().getName().equals(msgType))
      {
        Message msg                 = new IntegerMessage();
        read                       += msg.readMessage(is);
        ((Vector) super.message).addElement(msg);
      }
      else if(new LongMessage().getName().equals(msgType))
      {
        Message msg                 = new LongMessage();
        read                       += msg.readMessage(is);
        ((Vector) super.message).addElement(msg);
      }
      else if(new StringMessage().getName().equals(msgType))
      {
        Message msg                 = new StringMessage();
        read                       += msg.readMessage(is);
        ((Vector) super.message).addElement(msg);
      }
      else if(new VectorMessage().getName().equals(msgType))
      {
        Message msg                 = new VectorMessage();
        read                       += msg.readMessage(is);
        ((Vector) super.message).addElement(msg);
      }
    }

    super.messageHasBeenSet         = true;

    return read;
  }

  /**
   * Send current message throw given OutputStream
   * @param os The OutputStream used to read data
   * @return Number of wrote bytes
   * @throws java.io.IOException if an error occoured
   */
  public int writeMessage(OutputStream os) throws IOException
  {
    DataOutputStream dos;

    dos                             = new DataOutputStream(os);
    Read2J.writeLength(dos, ((Vector) super.message).size());
    int wrote                       = Read2J.PACKLENGTH;
    for(Enumeration e = ((Vector) super.message).elements(); e.hasMoreElements();)
    {
      MessageI msg                  = (MessageI) e.nextElement();
      wrote                        += msg.writeHeader(os);
      wrote                        += msg.writeMessage(os);
    }

    return wrote;
  }

  /**
   * Sets the internal object.
   * This function is <u>not</u> used in VectorMessage.
   * It's here just for compatibility.
   * @param obj Internal object to be set
   */
  protected void setInternalObject(Object obj)
  {
  }
}
